@@ -258,13 +258,13 @@ LEIM is available from the same ftp directory as Emacs."))
 
 (defvar quail-mode-map 
   (let ((map (make-keymap))
-	(i ? ))
+	(i (+ ?\  0)))
     (while (< i 127)
-      (define-key map (char-to-string i) 'quail-start-translation)
+      (define-key map (vector (list (int-char i))) 'quail-start-translation)
       (setq i (1+ i)))
     (setq i 128)
     (while (< i 256)
-      (define-key map (vector (int-char i)) 'quail-start-translation)
+      (define-key map (vector (list (int-char i))) 'quail-start-translation)
       (setq i (1+ i)))
     map)
   "Keymap for Quail mode.")
@@ -304,16 +304,16 @@ LEIM is available from the same ftp directory as Emacs."))
     (define-key map "\C-h" 'quail-translation-help)
 ;;; This interferes with handling of escape sequences on non-X terminals.
 ;;;    (define-key map "\e" '(keymap (t . quail-execute-non-quail-command)))
-    (define-key map [?\C-\ ] 'quail-select-current)
+    (define-key map [(control space)] 'quail-select-current)
     (define-key map [tab] 'quail-completion)
-    (define-key map [delete] 'quail-delete-last-char)
-    (define-key map [backspace] 'quail-delete-last-char)
+    (define-key map [delete] 'quail-backward-or-forward-delete-char)
+    (define-key map [backspace] 'quail-delete-backward-char)
     (let ((meta-map (make-sparse-keymap)))
       (when (characterp meta-prefix-char)
 	(define-key map (char-to-string meta-prefix-char) meta-map))
       (define-key map [escape] meta-map))
     (when (characterp meta-prefix-char)
-      (define-key map (vector meta-prefix-char t)
+      (define-key map (string meta-prefix-char ?t)
 	'quail-execute-non-quail-command))
     ;; At last, define default key binding.
     (set-keymap-default-binding map 'quail-execute-non-quail-command)
@@ -332,9 +332,8 @@ This map is activated while translation region is active.")
     (while (< i 127)
       (define-key map (char-to-string i) 'quail-self-insert-command)
       (setq i (1+ i)))
-    (define-key map "\177" 'quail-delete-last-char)
-    (define-key map [delete] 'quail-delete-last-char)
-    (define-key map [backspace] 'quail-delete-last-char)
+    (define-key map [delete] 'quail-backward-or-forward-delete-char)
+    (define-key map [backspace] 'quail-delete-backward-char)
 ;;; This interferes with handling of escape sequences on non-X terminals.
 ;;;    (define-key map "\e" '(keymap (t . quail-execute-non-quail-command)))
     (let ((meta-map (make-sparse-keymap)))
@@ -342,7 +341,7 @@ This map is activated while translation region is active.")
 	(define-key map (char-to-string meta-prefix-char) meta-map))
       (define-key map [escape] meta-map))
     (when (characterp meta-prefix-char)
-      (define-key map (vector meta-prefix-char t)
+      (define-key map (string meta-prefix-char ?t)
 	'quail-execute-non-quail-command))
     ;; At last, define default key binding.
     (set-keymap-default-binding map 'quail-execute-non-quail-command)
@@ -732,7 +731,7 @@ standard layout defined in `quail-keyboard-layout-standard'.  This
 function tells Quail system the layout of your keyboard so that what
 you type is correctly handled."
   (interactive
-   (let* ((completing-ignore-case t)
+   (let* ((completion-ignore-case t)
 	  (type (completing-read "Keyboard type: "
 				 quail-keyboard-layout-alist)))
      (list type)))
@@ -808,7 +807,7 @@ If it is a string, each character is a candidate for the translation.
 If it is a vector, each element (string or character) is a candidate
   for the translation.
 In these cases, a key specific Quail map is generated and assigned to KEY.
-(A Quail map is a cons that satisfies `quail-map-p', which see.)
+\(A Quail map is a cons that satisfies `quail-map-p', which see.)
 
 If TRANSLATION is a Quail map it is used to handle KEY.  
 
@@ -824,6 +823,13 @@ do, like manipulating unread-command-events, is not possible.  "
 	(while l
 	  (quail-defrule-internal (car (car l)) (car (cdr (car l))) map)
 	  (setq l (cdr l)))
+        (mapl
+         (function*
+          (lambda ((rule . rules))
+           (let ((assoc (assoc (car rule) rules)))
+             (and assoc
+                  (warn "rule %S overridden by %S" rule assoc)))))
+         rules)
 	map)))
 
 ;;;###autoload
@@ -1220,7 +1226,7 @@ The returned value is a Quail map specific to KEY."
 	      (maxcol (- (window-width quail-guidance-win)
 			 quail-guidance-translations-starting-column))
 	      (block (nth 3 indices))
-	      col idx width trans num-items blocks)
+	      col idx width trans num-items)
 	  (if (< cur start)
 	      ;; We must calculate from the head.
 	      (setq start 0 block 0)
@@ -1377,7 +1383,7 @@ sequence counting from the head."
   (quail-delete-region)
   (quail-terminate-translation))
 
-(defun quail-delete-last-char ()
+(defun quail-delete-backward-char ()
   "Delete the last input character from the current Quail key sequence."
   (interactive)
   (if (= (length quail-current-key) 1)
@@ -1385,6 +1391,24 @@ sequence counting from the head."
     (setq quail-current-key (substring quail-current-key 0 -1))
     (quail-update-translation (quail-translate-key))))
 
+(define-compatible-function-alias
+  'quail-delete-last-char
+  'quail-delete-backward-char)
+
+(defun quail-backward-or-forward-delete-char (arg)
+  "Delete either one input character backwards or one character forwards.
+
+Which direction depends on the variable `delete-key-deletes-forward' and
+whether the BackSpace keysym exists on your keyboard.  If there is no
+BackSpace keysym, this function always deletes one character backwards.
+
+With ARG (interactively, a prefix argument; see `universal-argument'),
+delete that number of characters, only if deleting forwards."
+  (interactive "*p")
+  (if (delete-forward-p)
+      (delete-char arg)
+    (quail-delete-backward-char)))
+
 ;; For conversion mode.
 
 (defun quail-conversion-backward-char ()
@@ -1442,6 +1466,12 @@ Remaining args are for FUNC."
 
 ;; Guidance, Completion, and Help buffer handlers.
 
+(defun-when-void frame-char-height (&optional frame)
+  "Return the height in pixels of a line of text on FRAME.
+
+If FRAME is on a TTY console, return the integer 1."
+  (face-height 'default (or frame (selected-frame))))
+
 ;; Make a new one-line frame for Quail guidance buffer.
 (defun quail-make-guidance-frame (buf)
   (let* ((fparam (frame-parameters))
@@ -1707,8 +1737,7 @@ All possible translations of the current key and whole possible longer keys
       (setq translations (cdr translations))
       ;; Insert every 10 elements with indices in a line.
       (let ((len (length translations))
-	    (i 0)
-	    num)
+	    (i 0))
 	(while (< i len)
 	  (when (zerop (% i 10))
 	    (when (>= i 10)
@@ -1825,7 +1854,7 @@ key		binding
 	(set-buffer standard-output)
 	(let ((quail-current-package package))
 	  (princ "You are translating the key sequence ")
-	  (prin1 quail-current-key)
+	  (prin1 current-key)
 	  (princ" in Quail mode.\n")
 	  (quail-help-insert-keymap-description
 	   (quail-translation-keymap)
@@ -1992,7 +2021,7 @@ of each directory."
   (interactive "FDirectory of LEIM: ")
   (setq dirname (expand-file-name dirname))
   (let ((leim-list (expand-file-name leim-list-file-name dirname))
-	quail-dirs list-buf pkg-list pkg-buf pos)
+	quail-dirs list-buf pkg-list pos)
     (if (not (file-writable-p leim-list))
 	(error "Can't write to file \"%s\"" leim-list))
     (message "Updating %s ..." leim-list)
